@@ -19,7 +19,7 @@
 #define LMARGIN 2
 #define RMARGIN 2
 /* for some reason, it draws text 1 pixel lower on Mac OS X (& linux too?) */
-#ifndef MSW
+#ifndef _WIN32
 #define TMARGIN 2
 #define BMARGIN 2
 #else
@@ -65,8 +65,6 @@ t_rtext *rtext_new(t_glist *glist, t_text *who)
     return (x);
 }
 
-static t_rtext *rtext_entered;
-
 void rtext_free(t_rtext *x)
 {
     if (x->x_glist->gl_editor->e_textedfor == x)
@@ -83,7 +81,6 @@ void rtext_free(t_rtext *x)
             break;
         }
     }
-    if (rtext_entered == x) rtext_entered = 0;
     freebytes(x->x_buf, x->x_bufsize);
     freebytes(x, sizeof *x);
 }
@@ -108,7 +105,7 @@ void rtext_getseltext(t_rtext *x, char **buf, int *bufsize)
 /* convert t_text te_type symbol for use as a Tk tag */
 static t_symbol *rtext_gettype(t_rtext *x)
 {
-    switch (x->x_text->te_type) 
+    switch (x->x_text->te_type)
     {
     case T_TEXT: return gensym("text");
     case T_OBJECT: return gensym("obj");
@@ -159,13 +156,13 @@ static int lastone(char *s, int c, int n)
         SEND_FIRST - draw the box  for the first time
         SEND_UPDATE - redraw the updated box
         otherwise - don't draw, just calculate.
-    Called with *widthp and *heightpas coordinates of
+    Called with *widthp and *heightp as coordinates of
     a test point, the routine reports the index of the character found
     there in *indexp.  *widthp and *heightp are set to the width and height
     of the entire text in pixels.
     */
 
-   /*-- moo: 
+   /*-- moo:
     * + some variables from the original version have been renamed
     * + variables with a "_b" suffix are raw byte strings, lengths, or offsets
     * + variables with a "_c" suffix are logical character lengths or offsets
@@ -181,11 +178,6 @@ static int lastone(char *s, int c, int n)
 #define UPBUFSIZE 4000
 #define BOXWIDTH 60
 
-/* Older (pre-8.3.4) TCL versions handle text selection differently; this
-flag is set from the GUI if this happens.  LATER take this out: early 2006? */
-
-extern int sys_oldtclversion;           
-
 static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
     int *indexp)
 {
@@ -206,10 +198,17 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
     if (pd_class(&x->x_text->te_pd) == canvas_class &&
         ((t_glist *)(x->x_text))->gl_isgraph &&
         ((t_glist *)(x->x_text))->gl_goprect)
-            font =  glist_getfont((t_glist *)(x->x_text));
-    else font = glist_getfont(x->x_glist);
-    fontwidth = sys_fontwidth(font);
-    fontheight = sys_fontheight(font);
+    {
+        font =  glist_getfont((t_glist *)(x->x_text));
+        fontwidth =  glist_fontwidth((t_glist *)(x->x_text));
+        fontheight =  glist_fontheight((t_glist *)(x->x_text));
+    }
+    else
+    {
+        font = glist_getfont(x->x_glist);
+        fontwidth = glist_fontwidth(x->x_glist);
+        fontheight = glist_fontheight(x->x_glist);
+    }
     findx = (*widthp + (fontwidth/2)) / fontwidth;
     findy = *heightp / fontheight;
     if (x->x_bufsize >= 100)
@@ -226,9 +225,13 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
         int foundit_c;
         if (foundit_b < 0)
         {
+                /* too much text to fit in one line? */
             if (inchars_c > widthlimit_c)
             {
-                foundit_b = lastone(x->x_buf + inindex_b, ' ', maxindex_b);
+                    /* is there a space to break the line at?  OK if it's even
+                    one byte past the end since in this context we know there's
+                    more text */
+                foundit_b = lastone(x->x_buf + inindex_b, ' ', maxindex_b + 1);
                 if (foundit_b < 0)
                 {
                     foundit_b = maxindex_b;
@@ -278,7 +281,7 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
     if (nlines < 1) nlines = 1;
     if (!widthspec_c)
     {
-        while (ncolumns < 3)
+        while (ncolumns < (x->x_text->te_type == T_TEXT ? 1 : 3))
         {
             tempbuf[outchars_b++] = ' ';
             ncolumns++;
@@ -288,43 +291,59 @@ static void rtext_senditup(t_rtext *x, int action, int *widthp, int *heightp,
     pixwide = ncolumns * fontwidth + (LMARGIN + RMARGIN);
     pixhigh = nlines * fontheight + (TMARGIN + BMARGIN);
 
+    if (action && x->x_text->te_width && x->x_text->te_type != T_ATOM)
+    {
+            /* if our width is specified but the "natural" width is the
+            same as the specified width, set specified width to zero
+            so future text editing will automatically change width.
+            Except atoms whose content changes at runtime. */
+        int widthwas = x->x_text->te_width, newwidth = 0, newheight = 0,
+            newindex = 0;
+        x->x_text->te_width = 0;
+        rtext_senditup(x, 0, &newwidth, &newheight, &newindex);
+        if (newwidth/fontwidth != widthwas)
+            x->x_text->te_width = widthwas;
+        else x->x_text->te_width = 0;
+    }
     if (action == SEND_FIRST)
+    {
         sys_vgui("pdtk_text_new .x%lx.c {%s %s text} %f %f {%.*s} %d %s\n",
             canvas, x->x_tag, rtext_gettype(x)->s_name,
             dispx + LMARGIN, dispy + TMARGIN,
-            outchars_b, tempbuf, sys_hostfontsize(font),
+            outchars_b, tempbuf,
+            sys_hostfontsize(font, glist_getzoom(x->x_glist)),
             (glist_isselected(x->x_glist,
                 &x->x_glist->gl_gobj)? "blue" : "black"));
+    }
     else if (action == SEND_UPDATE)
     {
         sys_vgui("pdtk_text_set .x%lx.c %s {%.*s}\n",
             canvas, x->x_tag, outchars_b, tempbuf);
-        if (pixwide != x->x_drawnwidth || pixhigh != x->x_drawnheight) 
+        if (pixwide != x->x_drawnwidth || pixhigh != x->x_drawnheight)
             text_drawborder(x->x_text, x->x_glist, x->x_tag,
                 pixwide, pixhigh, 0);
         if (x->x_active)
         {
             if (selend_b > selstart_b)
             {
-                sys_vgui(".x%lx.c select from %s %d\n", canvas, 
+                sys_vgui(".x%lx.c select from %s %d\n", canvas,
                     x->x_tag, u8_charnum(x->x_buf, selstart_b));
-                sys_vgui(".x%lx.c select to %s %d\n", canvas, 
-                    x->x_tag, u8_charnum(x->x_buf, selend_b)
-                              + (sys_oldtclversion ? 0 : -1));
-                sys_vgui(".x%lx.c focus \"\"\n", canvas);        
+                sys_vgui(".x%lx.c select to %s %d\n", canvas,
+                    x->x_tag, u8_charnum(x->x_buf, selend_b) - 1);
+                sys_vgui(".x%lx.c focus \"\"\n", canvas);
             }
             else
             {
                 sys_vgui(".x%lx.c select clear\n", canvas);
                 sys_vgui(".x%lx.c icursor %s %d\n", canvas, x->x_tag,
                     u8_charnum(x->x_buf, selstart_b));
-                sys_vgui(".x%lx.c focus %s\n", canvas, x->x_tag);        
+                sys_vgui(".x%lx.c focus %s\n", canvas, x->x_tag);
             }
         }
     }
     x->x_drawnwidth = pixwide;
     x->x_drawnheight = pixhigh;
-    
+
     *widthp = pixwide;
     *heightp = pixhigh;
     if (tempbuf != smallbuf)
@@ -392,9 +411,14 @@ t_rtext *glist_findrtext(t_glist *gl, t_text *who)
     t_rtext *x;
     if (!gl->gl_editor)
         canvas_create_editor(gl);
+    {
+        int i = 0;
+        for (x = gl->gl_editor->e_rtext; x && x->x_text != who; x = x->x_next)
+            i++;
+        // post("i=%d", i);
+    }
     for (x = gl->gl_editor->e_rtext; x && x->x_text != who; x = x->x_next)
         ;
-    if (!x) bug("glist_findrtext");
     return (x);
 }
 
@@ -425,7 +449,7 @@ void rtext_erase(t_rtext *x)
 
 void rtext_displace(t_rtext *x, int dx, int dy)
 {
-    sys_vgui(".x%lx.c move %s %d %d\n", glist_getcanvas(x->x_glist), 
+    sys_vgui(".x%lx.c move %s %d %d\n", glist_getcanvas(x->x_glist),
         x->x_tag, dx, dy);
 }
 
@@ -433,9 +457,8 @@ void rtext_select(t_rtext *x, int state)
 {
     t_glist *glist = x->x_glist;
     t_canvas *canvas = glist_getcanvas(glist);
-    sys_vgui(".x%lx.c itemconfigure %s -fill %s\n", canvas, 
+    sys_vgui(".x%lx.c itemconfigure %s -fill %s\n", canvas,
         x->x_tag, (state? "blue" : "black"));
-    canvas_editing = canvas;
 }
 
 void rtext_activate(t_rtext *x, int state)
@@ -486,7 +509,7 @@ void rtext_key(t_rtext *x, int keynum, t_symbol *keysym)
             if (x->x_selend < x->x_bufsize && (x->x_selstart == x->x_selend))
                 u8_inc(x->x_buf, &x->x_selend);
         }
-        
+
         ndel = x->x_selend - x->x_selstart;
         for (i = x->x_selend; i < x->x_bufsize; i++)
             x->x_buf[i- ndel] = x->x_buf[i];
@@ -519,8 +542,8 @@ be printable in whatever 8-bit character set we find ourselves. */
             int ch_nbytes = u8_wc_nbytes(n);
             newsize = x->x_bufsize + ch_nbytes;
             x->x_buf = resizebytes(x->x_buf, x->x_bufsize, newsize);
-            for (i = x->x_bufsize; i > x->x_selstart; i--)
-                x->x_buf[i] = x->x_buf[i-1];
+            for (i = newsize-1; i > x->x_selstart; i--)
+                x->x_buf[i] = x->x_buf[i-ch_nbytes];
             x->x_bufsize = newsize;
             /*-- moo: assume canvas_key() has encoded keysym as UTF-8 */
             strncpy(x->x_buf+x->x_selstart, keysym->s_name, ch_nbytes);
@@ -592,7 +615,7 @@ void rtext_mouse(t_rtext *x, int xval, int yval, int flag)
         if ((newseparator = lastone(x->x_buf, ',', indx)) > whereseparator)
             whereseparator = newseparator+1;
         x->x_selstart = whereseparator;
-        
+
         whereseparator = x->x_bufsize - indx;
         if ((newseparator =
             firstone(x->x_buf+indx, ' ', x->x_bufsize - indx)) >= 0 &&
